VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "WinHttpTime"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'===========
'WinHttpTime
'===========
'
'A class for formatting between Date values and commonly used Web
'Service timestamp strings using winhttp.dll routines.
'
'OS requirements
'---------------
'
'   Windows NT 4.0 with IE 5.01, or later.  No Win9x support.
'
'   Windows 2000 SP3 or later, Windows XP SP1 or later, Windows
'   Server 2003 SP1 or later, and any newer OS include winhttp.dll
'   as a system component.
'
'   The earlier OSs (back to NT 4.0) require separate install of
'   WinHttp 5.0 or 5.1 (5.1 not supported in NT 4.0) packages.
'
'NOTE
'----
'
'This may be a redundant class, since the similar class WinInetTime
'calls wininet.dll which the VB6 runtime probably always loads
'anyway.  Thus even in VB6 programs not explicitly using WinInet
'there may be no savings in using this version.
'
'PROPERTIES
'----------
'
'   None.
'
'METHODS
'-------
'
'   Public Function DateTimeToDate(ByVal DateTimeValue As String)
'       As Date
'
'           Accepts a String timestamp in ISO 8601 format and returns
'           a Date result adjusted to the current time zone.
'
'   Public Function DateToDateTime(ByVal DateValue As Date) As String
'
'           Accepts a Date timestamp for the current time zone and
'           returns a String in ISO 8601 format adjusted to Zulu Time
'           (GMT/UTC).
'
'   Public Function DateToHttpTime(ByVal DateValue As Date) As String
'
'           Accepts a Date timestamp for the current time zone and
'           returns a String in HTTP timestamp format adjusted to
'           Zulu Time (GMT/UTC).
'
'   Public Function HttpTimeToDate(ByVal HttpTimeValue As String)
'       As Date
'
'           Accepts a String timestamp in HTTP format and returns
'           a Date result adjusted to the current time zone.
'

Private Const WINHTTP_TIME_FORMAT_BUFSIZE As Long = 62

Private Const ZONES As String = _
    "Z,GMT,UTC,A,ACDT,ACST,ADT,AEDT,AEST,AKDT,AKST,AST,AWDT,AWST,B,BST,C,CDT,CEDT,CEST,CET," _
  & "CST,CXT,D,E,EDT,EEDT,EEST,EET,EST,F,G,H,HAA,HAC,HADT,HAE,HAP,HAR,HAST,HAT,HAY,HNA,HNC," _
  & "HNE,HNP,HNR,HNT,HNY,HST,I,IST,K,L,M,MDT,MESZ,MEZ,MSD,MSK,MST,N,NDT,NFT,NST,O,P,PDT,PST," _
  & "Q,R,S,T,U,V,W,WDT,WEDT,WEST,WET,WST,X,Y"
Private Const ZONEOFFSETS As String = _
    "0,0,0,1,10.5,9.5,-3,11,10,-8,-9,-4,9,8,2,1,3,-5,2,2,1,-6,7,4,5,-4,3,3,2,-5,6,7,8,-3,-5," _
  & "-9,-4,-7,-6,-10,-2.5,-8,-4,-6,-5,-8,-7,-3.5,-9,-10,9,1,10,11,12,-6,2,1,4,3,-7,-1,-2.5," _
  & "11.5,-3.5,-2,-3,-7,-8,-4,-5,-6,-7,-8,-9,-10,9,1,1,0,8,-11,-12"

Private Type TIME_ZONE_INFORMATION
    Bias As Long
    StandardName(0 To 31) As Integer
    StandardDate(0 To 7) As Integer
    StandardBias As Long
    DaylightName(0 To 31) As Integer
    DaylightDate(0 To 7) As Integer
    DaylightBias As Long
End Type

Private Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type

Private Declare Function GetTimeZoneInformation Lib "kernel32" ( _
    ByRef lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long

Private Declare Function SystemTimeToTzSpecificLocalTime Lib "kernel32" ( _
    ByVal lpTimeZoneInformation As Long, _
    ByVal lpUniversalTime As Long, _
    ByVal byreflpLocalTime As Long) As Long

Private Declare Function SystemTimeToVariantTime Lib "oleaut32" ( _
    ByVal lpSystemTime As Long, _
    ByRef pvtime As Date) As Long 'Non-zero = success.

Private Declare Function TzSpecificLocalTimeToSystemTime Lib "kernel32" ( _
    ByVal lpTimeZoneInformation As Long, _
    ByVal lpLocalTime As Long, _
    ByVal lpUniversalTime As Long) As Long

Private Declare Function VariantTimeToSystemTime Lib "oleaut32" ( _
    ByVal vtime As Date, _
    ByVal lpSystemTime As Long) As Long 'Non-zero = success.
    
Private Declare Function WinHttpTimeFromSystemTime Lib "winhttp" ( _
    ByVal pst As Long, _
    ByVal pwszTime As Long) As Long 'Non-zero = success

Private Declare Function WinHttpTimeToSystemTime Lib "winhttp" ( _
    ByVal pwszTime As Long, _
    ByVal pst As Long) As Long 'Non-zero = success.

Private TZones() As String
Private TZoneOffsets() As String

Public Function DateTimeToDate(ByVal DateTimeValue As String) As Date
    'These timestamps are in ISO 8601 date format.
    '
    '   YYYY-MM-DDTHH:MM:SS.F+HH:MM
    'or:
    '   YYYY-MM-DDTHH:MM:SS.F+HHMM
    'or:
    '   YYYY-MM-DDTHH:MM:SS.FZ  (Zulu a.k.a. GMT/UTC)
    '
    '... and the ".F" fractions of seconds is optional!  We'll just
    'truncate/ignore it if present since Date doesn't have enough
    'precision.
    Dim TZI As TIME_ZONE_INFORMATION
    Dim DTLen As Integer
    Dim ISOOffset As Single
    Dim PlusAt As Integer
    
    GetTimeZoneInformation TZI 'Must call each time since it might change.
    DTLen = Len(DateTimeValue)
    Mid$(DateTimeValue, 11, 1) = " " 'Change the "T" to " ".
    If Right$(DateTimeValue, 1) <> "Z" Then
        PlusAt = IIf(Mid$(DateTimeValue, 2, 1) = ":", -5, -4)
        ISOOffset = IIf(Mid$(DateTimeValue, DTLen - PlusAt, 1) = "+", 1#, -1#) _
                  * (CSng(Mid$(DateTimeValue, DTLen - (PlusAt + 1), 2)) _
                   + CSng(Mid$(DTLen - 1, 2)) / 60#)
    End If
    DateTimeToDate = DateAdd("h", _
                             ISOOffset - CSng(TZI.Bias) / 60#, _
                             CDate(Left$(DateTimeValue, 19)))
End Function

Public Function DateToDateTime(ByVal DateValue As Date) As String
    'These timestamps are in ISO 8601 date format.
    Dim TZI As TIME_ZONE_INFORMATION

    GetTimeZoneInformation TZI 'Must call each time since it might change.
    DateToDateTime = Format$(DateAdd("h", CSng(TZI.Bias) / 60#, DateValue), _
                             "yyyy-mm-dd\THh:Nn:Ss\Z")
End Function

Public Function DateToHttpTime(ByVal DateValue As Date) As String
    Dim ST As SYSTEMTIME
    Dim HT As String
    
    If VariantTimeToSystemTime(DateValue, VarPtr(ST)) = 0 Then
        Err.Raise vbObjectError Or &H9E12&, _
                  "WinHttpTime.DateToHttpTime", _
                  "VariantTimeToSystemTime failed, system error " _
                & CStr(Err.LastDllError)
    ElseIf TzSpecificLocalTimeToSystemTime(0, VarPtr(ST), VarPtr(ST)) = 0 Then
        Err.Raise vbObjectError Or &H9E14&, _
                  "WinHttpTime.DateToHttpTime", _
                  "TzSpecificLocalTimeToSystemTime failed, system error " _
                & CStr(Err.LastDllError)
    Else
        HT = String$(WINHTTP_TIME_FORMAT_BUFSIZE, 0)
        If WinHttpTimeFromSystemTime(VarPtr(ST), StrPtr(HT)) = 0 Then
            Err.Raise vbObjectError Or &H9E16&, _
                      "WinHttpTime.DateToHttpTime", _
                      "WinHttpTimeFromSystemTime failed, system error " _
                    & CStr(Err.LastDllError)
        Else
            DateToHttpTime = Left$(HT, InStr(HT, vbNullChar) - 1)
        End If
    End If
End Function

Public Function HttpTimeToDate(ByVal HttpTimeValue As String) As Date
    Dim ST As SYSTEMTIME
    Dim DT As Date
    
    If WinHttpTimeToSystemTime(StrPtr(HttpTimeValue), VarPtr(ST)) = 0 Then
        Err.Raise vbObjectError Or &H9E22&, _
                  "WinHttpTime.HttpTimeToDate", _
                  "WinHttpTimeToSystemTime failed, system error " _
                & CStr(Err.LastDllError)
    ElseIf SystemTimeToTzSpecificLocalTime(0, VarPtr(ST), VarPtr(ST)) = 0 Then
        Err.Raise vbObjectError Or &H9E24&, _
                  "WinHttpTime.HttpTimeToDate", _
                  "SystemTimeToTzSpecificLocalTime failed, system error " _
                & CStr(Err.LastDllError)
    ElseIf SystemTimeToVariantTime(VarPtr(ST), DT) = 0 Then
        Err.Raise vbObjectError Or &H9E26&, _
                  "WinHttpTime.HttpTimeToDate", _
                  "SystemTimeToVariantTime failed, system error " _
                & CStr(Err.LastDllError)
    Else
        HttpTimeToDate = DT
    End If
End Function

Private Sub Class_Initialize()
    TZones = Split(ZONES, ",")
    TZoneOffsets = Split(ZONEOFFSETS, ",")
End Sub

